@@ -10,38 +10,58 @@ Dancer::Plugin::Database - easy database connections for Dancer applications
 
 =cut
 
-our $VERSION = '0.08';
+our $VERSION = '0.09';
 
-my $dbh;
-my $last_connection_check;
 my $settings = plugin_setting;
-$settings->{connection_check_threshold} ||= 30;
+my %handles;
+# Hashref used as key for default handle, so we don't have a magic value that
+# the user could use for one of their connection names and cause problems
+# (Kudos to Igor Bujna for the idea)
+my $def_handle = {};
 
 register database => sub {
-    if ($dbh) {
-        if (time - $last_connection_check
+    my $name = shift;
+    my $handle = defined($name) ? $handles{$name} : $def_handle;
+    my $settings = _get_settings($name);
+
+    if ($handle->{dbh}) {
+        if (time - $handle->{last_connection_check}
             < $settings->{connection_check_threshold}) {
-            return $dbh;
+            return $handle->{dbh};
         } else {
-            if (_check_connection($dbh)) {
-                $last_connection_check = time;
-                return $dbh;
+            if (_check_connection($handle->{dbh})) {
+                $handle->{last_connection_check} = time;
+                return $handle->{dbh};
             } else {
-                Dancer::Logger->debug(
+                Dancer::Logger::debug(
                     "Database connection went away, reconnecting"
                 );
-                if ($dbh) { $dbh->disconnect; }
-                return $dbh = _get_connection();
+                if ($handle->{dbh}) { $handle->{dbh}->disconnect; }
+                return $handle->{dbh}= _get_connection();
             }
         }
     } else {
-        return $dbh = _get_connection();
+        # Get a new connection
+        if (!$settings) {
+            Dancer::Logger::error(
+                "No DB settings named $name, so cannot connect"
+            );
+            return;
+        }
+        if ($handle->{dbh} = _get_connection($settings)) {
+            $handle->{last_connection_check} = time;
+            return $handle->{dbh};
+        } else {
+            return;
+        }
     }
 };
 
 register_plugin;
 
+# Given the settings to use, try to get a database connection
 sub _get_connection {
+    my $settings = shift;
 
     # Assemble the DSN:
     my $dsn;
@@ -53,8 +73,10 @@ sub _get_connection {
 
         # DBD::SQLite wants 'dbname', not 'database', so special-case this
         # (DBI's documentation recommends that DBD::* modules should understand
-        # 'database', but DBD::SQLite doesn't; let's make things easier for our
-        # users by handling this for them):
+        # 'database', but older versions of DBD::SQLite didn't; let's make 
+        # things easier for our users by handling this for them):
+        # (DBD::SQLite will support 'database', too, as of 1.32 when it's
+        # released)
         if ($settings->{driver} eq 'SQLite' 
             && $settings->{database} && !$settings->{dbname}) {
             $settings->{dbname} = delete $settings->{database};
@@ -73,18 +95,17 @@ sub _get_connection {
     );
 
     if (!$dbh) {
-        Dancer::Logger->error(
+        Dancer::Logger::error(
             "Database connection failed - " . $DBI::errstr
         );
     } elsif (exists $settings->{on_connect_do}) {
         for (@{ $settings->{on_connect_do} }) {
-            $dbh->do($_) or Dancer::Logger->error(
+            $dbh->do($_) or Dancer::Logger::error(
                 "Failed to perform on-connect command $_"
             );
         }
     }
 
-    $last_connection_check = time;
     return $dbh;
 }
 
@@ -112,6 +133,41 @@ sub _check_connection {
     }
 }
 
+sub _get_settings {
+    my $name = shift;
+    my $return_settings;
+
+    # If no name given, just return the default settings
+    # (Take a copy and remove the connections key, so we have only the main
+    # connection details)
+    if (!defined $name) {
+        $return_settings = { %$settings };
+    } else {
+        # If there are no named connections in the config, bail now:
+        return unless exists $settings->{connections};
+
+
+        # OK, find a matching config for this name:
+        if (my $settings = $settings->{connections}{$name}) {
+            $return_settings = { %$settings };
+        } else {
+            # OK, didn't match anything
+            Dancer::Logger::error(
+                "Asked for a database handle named '$name' but no matching  "
+               ."connection details found in config"
+            );
+        }
+    }
+    
+    # We should have soemthing to return now; remove any unrelated connections
+    # (only needed if this is the default connection), and make sure we have a
+    # connection_check_threshold, then return what we found
+    delete $return_settings->{connections};
+    $return_settings->{connection_check_threshold} ||= 30;
+    return $return_settings;
+
+}
+
 
 =head1 SYNOPSIS
 
@@ -185,6 +241,31 @@ in that case, it will be used as-is, and the driver/database/host settings will
 be ignored.  This may be useful if you're using some DBI driver which requires 
 a peculiar DSN.
 
+
+=head2 DEFINING MULTIPLE CONNECTIONS
+
+If you need to connect to multiple databases, this is easy - just list them in
+your config under C<connections> as shown below:
+
+    plugins:
+        Database:
+            connections:
+                foo:
+                    driver: "SQLite"
+                    database: "foo.sqlite"
+                bar:
+                    driver: "mysql"
+                    host: "localhost"
+                    ....
+
+Then, you can call the C<database> keyword with the name of the database
+connection you want, for example:
+
+    my $foo_dbh = database('foo');
+    my $bar_dbh = database('bar');
+
+
+
 =head1 GETTING A DATABASE HANDLE
 
 Calling C<database> will return a connected database handle; the first time it is
@@ -193,6 +274,10 @@ reference to the DBI object.  On subsequent calls, the same DBI connection
 object will be returned, unless it has been found to be no longer usable (the
 connection has gone away), in which case a fresh connection will be obtained.
 
+If you have declared named connections as described above in 'DEFINING MULTIPLE
+CONNECTIONS', then calling the database() keyword with the name of the
+connection as specified in the config file will get you a database handle
+connected with those details.
 
 =head1 AUTHOR
 
@@ -209,6 +294,11 @@ L<http://github.com/bigpresh/Dancer-Plugin-Database>
 Feel free to fork the repo and submit pull requests!
 
 
+=head1 ACKNOWLEDGEMENTS
+
+Igor Bujna
+
+
 =head1 BUGS
 
 Please report any bugs or feature requests to C<bug-dancer-plugin-database at rt.cpan.org>, or through
@@ -247,6 +337,8 @@ L<http://search.cpan.org/dist/Dancer-Plugin-Database/>
 
 =back
 
+You can find the author on IRC in the channel C<#dancer> on <irc.perl.org>.
+
 
 =head1 LICENSE AND COPYRIGHT
 
